@@ -16,7 +16,7 @@ except ImportError:
 if sys.platform == 'win32':
     try:
         import ctypes
-        PROCESS_SYSTEM_DPI_AWARE = 1  # Int required.
+        PROCESS_SYSTEM_DPI_AWARE = 1
         ctypes.OleDLL('shcore').SetProcessDpiAwareness(PROCESS_SYSTEM_DPI_AWARE)
     except (ImportError, AttributeError, OSError):
         pass
@@ -676,6 +676,7 @@ class ModifiedInterpreter(InteractiveInterpreter):
     def runsource(self, source):
         "Extend base class method: Stuff the source in the line cache first"
         filename = self.stuffsource(source)
+        self.more = 0
         # at the moment, InteractiveInterpreter expects str
         assert isinstance(source, str)
         # InteractiveInterpreter.runsource() calls its runcode() method,
@@ -992,12 +993,12 @@ class PyShell(OutputWindow):
     def beginexecuting(self):
         "Helper for ModifiedInterpreter"
         self.resetoutput()
-        self.executing = True
+        self.executing = 1
 
     def endexecuting(self):
         "Helper for ModifiedInterpreter"
-        self.executing = False
-        self.canceled = False
+        self.executing = 0
+        self.canceled = 0
         self.showprompt()
 
     def close(self):
@@ -1074,7 +1075,7 @@ class PyShell(OutputWindow):
     def readline(self):
         save = self.reading
         try:
-            self.reading = True
+            self.reading = 1
             self.top.mainloop()  # nested mainloop()
         finally:
             self.reading = save
@@ -1086,11 +1087,11 @@ class PyShell(OutputWindow):
             line = "\n"
         self.resetoutput()
         if self.canceled:
-            self.canceled = False
+            self.canceled = 0
             if not use_subprocess:
                 raise KeyboardInterrupt
         if self.endoffile:
-            self.endoffile = False
+            self.endoffile = 0
             line = ""
         return line
 
@@ -1108,8 +1109,8 @@ class PyShell(OutputWindow):
             self.interp.write("KeyboardInterrupt\n")
             self.showprompt()
             return "break"
-        self.endoffile = False
-        self.canceled = True
+        self.endoffile = 0
+        self.canceled = 1
         if (self.executing and self.interp.rpcclt):
             if self.interp.getdebugger():
                 self.interp.restart_subprocess()
@@ -1129,8 +1130,8 @@ class PyShell(OutputWindow):
             self.resetoutput()
             self.close()
         else:
-            self.canceled = False
-            self.endoffile = True
+            self.canceled = 0
+            self.endoffile = 1
             self.top.quit()
         return "break"
 
@@ -1302,7 +1303,7 @@ class PyShell(OutputWindow):
             raise ###pass  # ### 11Aug07 KBK if we are expecting exceptions
                            # let's find out what they are and be specific.
         if self.canceled:
-            self.canceled = False
+            self.canceled = 0
             if not use_subprocess:
                 raise KeyboardInterrupt
         return count
@@ -1485,14 +1486,9 @@ def main():
         iconfile = os.path.join(icondir, 'idle.ico')
         root.wm_iconbitmap(default=iconfile)
     elif not macosx.isAquaTk():
-        if TkVersion >= 8.6:
-            ext = '.png'
-            sizes = (16, 32, 48, 256)
-        else:
-            ext = '.gif'
-            sizes = (16, 32, 48)
+        ext = '.png' if TkVersion >= 8.6 else '.gif'
         iconfiles = [os.path.join(icondir, 'idle_%d%s' % (size, ext))
-                     for size in sizes]
+                     for size in (16, 32, 48)]
         icons = [PhotoImage(master=root, file=iconfile)
                  for iconfile in iconfiles]
         root.wm_iconphoto(True, *icons)
